"""Adapters to send notification with different mediums."""
import abc

class BaseAdapter(metaclass=abc.ABCMeta):
    """Base adapter for sending notifications."""

    def __init__(self, **kwargs):
        self.kwargs = kwargs

    @abc.abstractmethod
    def construct_message(self):
        """Constructs a message from notification details."""
        pass

    @abc.abstractmethod
    def notify(self):
        """Sends the notification."""
        pass



"""Defines and listens to notification signals."""

import importlib

from django.dispatch import Signal, receiver
from django.conf import settings

from .models import Notification
from . import NotificationError

notify = Signal(providing_args=('user', 'actor', 'action' 'obj', 'url'))
read = Signal(providing_args=('notify_id', 'recipent'))


def import_attr(path):
    """helper to import classes/attributes from str paths."""
    package, attr = path.rsplit('.', 1)

    return getattr(importlib.import_module(package), attr)

@receiver(notify)
def create_notification(sender, **kwargs):
    """Notify signal receiver."""
    # make fresh copy and retain kwargs
    params = kwargs.copy()
    del params['signal']

    Notification.objects.create(**params)

    # send via custom adapters
    for adapter_path in settings.NOTIFICATION_ADAPTERS:
        adapter = import_attr(adapter_path)
        adapter(**kwargs).notify()

@receiver(read)
def read_notification(sender, **kwargs):
    """
    Mark notification as read.

    Raises NotificationError if the user doesn't have access
    to read the notification
    """
    notify_id = kwargs['notify_id']
    recipent = kwargs['recipent']
    notification = Notification.objects.get(id=notify_id)

    if recipent != notification.recipent:
        raise NotificationError('You cannot read this notification')

    notification.read()



from django.test import TestCase
from django.contrib.auth import get_user_model

from .models import Notification
from .signals import read
from . import NotificationError


User = get_user_model()


class NotificationSignalTestCase(TestCase):
    """Tests for the notifications app."""

    @classmethod
    def setUpTestData(cls):
        """Create Users."""
        cls.user1 = User.objects.create_user(
            username='user1@gmail.com', password='password'
            )

        cls.user2 = User.objects.create(
            username='user2@gmail.com', password='password'
            )

    def test_user_cant_read_others_notifications(self):
        """A user should only be able to read THEIR notifications."""
        # Create NOtification for User2
        notification = Notification.objects.create(
            source=self.user1, source_display_name='User 1',
            recipent=self.user2, action='Notified',
            category='General notification', obj=1, url='http://example.com',
            is_read=False
        )

        # Try and Read the notification as User1
        self.assertRaises(
            NotificationError,

            read.send,
            sender=self.__class__, notify_id=notification.id,
            recipent=self.user1
            )

    def test_user_can_read_notifications(self):
        """A user can read their notification"""
        # Create NOtification for User1
        notification = Notification.objects.create(
            source=self.user2, source_display_name='User 2',
            recipent=self.user1, action='Notified',
            category='General notification', obj=1, url='http://example.com',
            is_read=False
        )

        # Try and Read the notification as user1
        read.send(
            sender=self.__class__, notify_id=notification.id,
            recipent=self.user1
            )

        notification.refresh_from_db()
        self.assertEqual(notification.is_read, True)



#!/usr/bin/env python

import os
from setuptools import setup

def package_files(directory):
    """Recursively add subfolders and files."""
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join('..', path, filename))
    return paths

extra_files = package_files('notifications')

setup(name='django-notifs',
      version='1.9',
      description='Re-usable notification app for Django',
      url='https://github.com/danidee10/django-notifs',
      author='Osaetin Daniel',
      author_email='osaetindaniel@gmail.com',
      license='GPL',
      packages=['notifications'],
      package_data={'': extra_files},
      install_requires=[
          'django',
      ],
      zip_safe=False)
